/*******************************************************************************
 *    Temporizador_3h_tool.c
 *
 *    Autor: Cludio Lrios
 *
 *    Data:22/11/2015
 *
 *    Temporizador de 3 horas para desligamento automtico de carga de bateria.
 *    Usado em ferramentas eltricas como parafusadeiras,furadeiras, etc.
 *    Objetivo: Evitar excessos de carga na bateria por esquecimento do usurio
 *    Mesmo que acabe a energia da rede momentaneamente, ao retornar, continua
 *    a contar o tempo restante. Os minutos so salvos na EEprom assim como a
 *    situao ligada/desligada.
 *
 *    Use CCS C Compiler
 *
 *    Uso didtico.
 *    Este arquivo  parte integrante do blog Lrios.Tecnologia.WS
 *
 *******************************************************************************/

#include <12F675.h>
#device adc=8
#use delay(clock=4000000)// OSC INTERNO DE 4MHZ
#fuses NOWDT,HS, NOCPD, NOPROTECT, NOMCLR, PUT, NOBROWNOUT //CONFIG WORD


#define XTAL  4000000  //frequencia do cristal usado
#define presc  64 // valor do prescaller do timer usado (8bits)
#define fator   (256*presc*4)
#define  sw      GPIO3    //pino 4  interruptor de presso on/off/restart
#define  out     GPIO2    //pino 5  sada para ligar/desligar carga via transistores
#define  beep    GPIO1    //pino 6  sada de som para disco piezoeletrico (beep)
#define  tempo_ligado 180 // em minutos



#byte   GPIO        = 0x05
#byte   TRISIO      = 0x85
#byte   TMR1H       = 0x0F
#byte   TMR1L       = 0x0E
#byte   TMR0        = 0X01

#byte   OPTION_REG  = 0x81
#byte   WPU         = 0x95
#byte   CMCON       = 0x19
#byte   ANSEL       = 0x9F
#byte   T1CON       = 0X10



#bit    GIE     = 0xb.7
#bit    PEIE    = 0xb.6
#bit    T0IE    = 0xb.5
#bit    TMR1IE  = 0x8C.0
#bit    TMR1IF  = 0xC.0
#bit    T0IF    = 0XB.2
#bit    GPIO0   = 0X5.0
#bit    GPIO1   = 0X5.1
#bit    GPIO2   = 0X5.2
#bit    GPIO3   = 0X5.3

//VARIAVEIS
unsigned int32  cnt32=0;
unsigned int8 cnt,seg,min;
int1 f_on, f_at;

//EEprom
#rom 0x2100 = {0,0}//situao inicial da eeprom ps programao do pic




/*==============================================================================
       ROTINA QUE BUSCA VALOR DE MINUTOS GRAVADO EM POSIES DIFERENTES DA
       EEPROM - (AUMENTAR VIDA UTIL DA EEPROM)
==============================================================================*/

unsigned int8 pega_min(){ // valor de minutos poder estar entre posio 1 e 63
   unsigned int8 a;
   for(a=1;a<127;a++){
    if(read_eeprom(a)!=0xff) return (a);//retorna posio onde est 'min'

   }
 }
/*==============================================================================
       ROTINA QUE GRAVA VALOR DE MINUTOS EM POSIES DIFERENTES DA
       EEPROM - (AUMENTAR VIDA UTIL DA EEPROM)
==============================================================================*/

void grava_min(unsigned int8 b){// valor de minutos poder estar entre posio 1 e 63
  if(b<127) {
     write_eeprom((b+1),min);//grava a prxima posio
     write_eeprom (b,0xff);}//apaga a anterior
  else{
      write_eeprom(1,min);//grava no inicio do espao reservado para minutos
    }

}

/*==============================================================================
       ROTINA GERADORA DE SOM INDICADOR DE FIM DE CARGA (BEEP) 
==============================================================================*/
void bipa(){
int16 a;
int8 b;
  for(b=0;b<5;b++){
   for(a=0;a<1000;a++){
   
     beep=1;
     delay_us(100);
     beep=0;
     delay_us(100);
   
   }
   delay_ms(700);
   
  }

}



/*==============================================================================
       ROTINA DE INTERRUPO DO TIMER 0 (OBTEM TEMPO DE 1 SEGUNDO)
==============================================================================*/
#INT_TIMER0
void  isr_tmr_0(){

   T0IF=0;//reseta flag over flow do timer 0
   cnt32+=fator;
   if(cnt32>=XTAL){
        cnt32-=XTAL;
        if(++seg>59) {
          seg=0;
          ++min;
          f_at=1;
       //  grava_min(pega_min());//grava  minutos na eeprom
        }
         GPIO0=!GPIO0;//inverte pino 7 a cada segundo
  }
}
/*==============================================================================
      ROTINA  MAIN
==============================================================================*/


void main() {

   OPTION_REG=5;//TIMER0  COM PRESCALLER /64
   CMCON=7; //desliga comparadores
   ANSEL=0;//desliga AD
   TMR0=131;//PRE-CARGA INICIAL
   cnt=0;//ZERA CONTADOR
   T0IF=0;//reseta flag do tmr0
   T0IE=1;//liga interrupo do timer 0
   GIE=1;//liga chave geral das interrupes
   GPIO=0;//desliga as sadas
   TRISIO=0;//tudo como sada  (exceto GP3)
   f_at=0;//flag atualizar eeprom
   min=  read_eeprom(pega_min());//recupera valor dos minutos na falta de energia

   if(read_eeprom(0)){ f_on=1;}//deterina se estava ligado ou desligado
   else{ f_on=0;}

/*==============================================================================
        LAO PRINCIPAL DE REPETIO
==============================================================================*/

    for(;;){
    
      if(f_at){
        GIE=0;//desliga interrupo geral
        f_at=0;
        grava_min(pega_min());//grava  minutos na eeprom
        GIE=1;//religa interrupo geral
      }
      if(!sw){//testa interruptor on/off/ restart
          GIE=0;//desliga interrupes
          if(f_on) {//se estiver ligado, ir desligar a carga
              f_on=0; //reseta
              write_eeprom(0,0);//salva estado de desligado
              min=0;//zera contador
              seg=0;//idem
              grava_min(pega_min());//grava zero nos minutos na eeprom


          }
          else{//se estiver desligado, ir ligar a carga
              f_on=1; //seta
              write_eeprom(0,1);//salva condio de ligado na eeprom
              GIE=1;//liga interrupo para comear a contar tempo
           }

              delay_ms(100);//delay
              while(!sw)delay_ms(50);//aguarda solta interruptor on/off/restart

        }

      if(f_on){//testa flag de on/off
         out=1;//liga carga

         if(min>=tempo_ligado) {
             GIE=0;//desliga interrupes
             min=0;//zera contador
             seg=0;//idem
             grava_min(pega_min());//grava zero nos minutos na eeprom
             write_eeprom(0,0);//salva condio de desligado
             f_on=0;//reseta
             bipa();

         }

      }
      else{ GPIO=0;  }//desliga sada da carga

    }  //for(;;)


}//main

