/******************************************************************************* 
 *    Temporizador_3h_tool.c
 * 
 *    Autor: Cludio Lrios
 *
 *    Data:22/11/2015
 * 
 *    Temporizador de 3 horas para desligamento automtico de carga de bateria. 
 *    Usado em ferramentas eltricas como parafusadeiras,furadeiras, etc.
 *    Objetivo: Evitar excessos de carga na bateria por esquecimento do usurio
 *    Mesmo que acabe a energia da rede momentaneamente, ao retornar, continua
 *    a contar o tempo restante. Os minutos so salvos na EEprom e como a situa-
 *    o ligada/desligada.
 * 
 *    Use MplabX (XC8 Compiler)
 * 
 *    Uso didtico.
 *    Este arquivo  parte integrante do blog Lrios.Tecnologia.WS
 *    
 *******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <xc.h>
#define XTAL 4000000  //frequencia do cristal usado 
#define _XTAL_FREQ  XTAL
#define presc  64 // valor do prescaller do timer usado (8bits)
#define fator  (long) 256*presc*4 
#define  sw      GPIO3    //pino 4  interruptor de presso on/off/restart
#define  out     GPIO2    //pino 5  sada para ligar/desligar carga via transistores
#define  beep    GPIO1    //pino 6  sada de som para disco piezoeletrico (beep)
#define  tempo_ligado 180 // em minutos


// CONFIG
#pragma config FOSC = HS   //Oscillator Selection bits (HS oscillator: Crystal/resonator on GP4/OSC2/CLKOUT and GP5/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON      // Power-Up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = OFF      // GP3/MCLR pin function select (GP3/MCLR pin function is digital I/O, MCLR internally tied to VDD)
#pragma config BOREN = ON       // Brown-out Detect Enable bit (BOD enabled)
#pragma config CP = OFF         // Code Protection bit (Program Memory code protection is disabled)
#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)

//VARIAVEIS
long  cnt32=0;
char cnt,seg,min;
bit f_on,f_at;
//EEprom
__EEPROM_DATA(0,0,0xff,0xff,0xff,0xff,0xff,0xff);//situao inicial da eeprom ps programao do pic



/*==============================================================================
       ROTINA QUE BUSCA VALOR DE MINUTOS GRAVADO EM POSIES DIFERENTES DA
       EEPROM - (AUMENTAR VIDA UTIL DA EEPROM)
==============================================================================*/

char pega_min(){ // valor de minutos poder estar entre posio 1 e 127
   char a;
   for(a=1;a<127;a++){
    if(eeprom_read(a)!=0xff) return (a);//retorna posio onde est 'min'

   }
 }
/*==============================================================================
       ROTINA QUE GRAVA VALOR DE MINUTOS EM POSIES DIFERENTES DA
       EEPROM - (AUMENTAR VIDA UTIL DA EEPROM)
==============================================================================*/

void grava_min(char b){// valor de minutos poder estar entre posio 1 e 127
  if(b<127) {
     eeprom_write((b+1),min);//grava a prxima posio
     eeprom_write (b,0xff);}//apaga a anterior
  else{
      eeprom_write(1,min);//grava no inicio do espao reservado para minutos
    }

}
/*==============================================================================
       ROTINA GERADORA DE SOM INDICADOR DE FIM DE CARGA (BEEP) 
==============================================================================*/
void bipa(){
int a;
char b;
  for(b=0;b<5;b++){
   for(a=0;a<1000;a++){
   
     beep=1;
     __delay_us(100);
     beep=0;
     __delay_us(100);
   
   }
   __delay_ms(700);
   
  }

}


/*==============================================================================
       ROTINA DE INTERRUPO DO TIMER 0 (OBTEM TEMPO DE 1 SEGUNDO)
==============================================================================*/
void interrupt isr_tmr_0(){
  
   T0IF=0;//reseta flag over flow do timer 0
   cnt32+=fator;
   if(cnt32>=XTAL){ 
        cnt32-=XTAL;
        if(++seg>59) {
          seg=0;
          ++min;
          f_at=1;
         //grava_min(pega_min());//grava  minutos na eeprom
        }
         GPIO0=!GPIO0;//inverte pino 7 a cada segundo
  }
} 
/*==============================================================================
      ROTINA  MAIN
==============================================================================*/


int main(int argc, char** argv) {

   OPTION_REG=5;//TIMER0  COM PRESCALLER /64
   CMCON=7; //desliga comparadores
   ANSEL=0;//desliga AD
   TMR0=131;//PRE-CARGA INICIAL
   cnt=0;//ZERA CONTADOR
   T0IF=0;//reseta flag do tmr0
   T0IE=1;//liga interrupo do timer 0
   GIE=1;//liga chave geral das interrupes
   GPIO=0;//desliga as sadas
   TRISIO=0;//tudo como sada  (exceto GP3) 
   f_at=0;
     
   min=  eeprom_read(pega_min());//recupera valor dos minutos na falta de energia
   
   if(eeprom_read(0)){ f_on=1;}//deterina se estava ligado ou desligado
   else{ f_on=0;}
   
/*==============================================================================
        LAO PRINCIPAL DE REPETIO
==============================================================================*/  
    
    for(;;){
        if(f_at){
            f_at=0;
            grava_min(pega_min());//grava  minutos na eeprom
        }
      if(!sw){//testa interruptor on/off/ restart
          GIE=0;//desliga interrupes
          if(f_on) {//se estiver ligado, ir desligar a carga
              f_on=0; //reseta
              eeprom_write(0,0);//salva estado de desligado
              min=0;//zera contador
              seg=0;//idem
              grava_min(pega_min());//grava  minutos na eeprom
              
             
          }
          else{//se estiver desligado, ir ligar a carga
              f_on=1; //seta
              eeprom_write(0,1);//salva condio de ligado na eeprom
              GIE=1;//liga interrupo para comear a contar tempo
           }
              
              __delay_ms(100);//delay
              while(!sw)__delay_ms(50);//aguarda solta interruptor on/off/restart
              
        }
    
      if(f_on){//testa flag de on/off
         out=1;//liga carga
         
         if(min>=tempo_ligado) {
             GIE=0;//desliga interrupes
             min=0;//zera contador
             seg=0;//idem
             grava_min(pega_min());//grava zero nos minutos na eeprom
             eeprom_write(0,0);//salva condio de desligado 
             f_on=0;//reseta
             bipa();
         }
      
      }
      else{ GPIO=0;  }//desliga sada da carga
      
    }  //for(;;)
   
        return (EXIT_SUCCESS);
}//main

